-- used compiler flags:
-- ghc --make -O -fglasgow-exts BenchJtoS.hs
--
-- small benchmark comparing the speed of the old and the new
-- jtos implementations.

module Main (main) where

import GHC.Base (ord)
import qualified JtoSnew  as N
import qualified JtoSold  as O

import System.CPUTime (getCPUTime)

-- test printing with each function
-- input: a list of numbers
-- prints the time used for converting each of the numbers to a string
-- using N.jtos and O.jtos.

test :: String -> [Integer] -> IO ()
test name l = do
    -- print test name
    putStrLn $ name
    -- first evaluate all the numbers to get a fair comparison
    (sum (map (`mod` 3) l) == 0) `seq` return ()
    -- measure N.jtos time
    x <- getCPUTime
    putStr $ show $ sum $ map ord $ foldr N.jtos  "" l
    y <- getCPUTime
    putStrLn $ " --> (N.jtos) time=" ++ show (y-x)
    -- measure O.jtos time
    x <- getCPUTime
    putStr $ show $ sum $ map ord $ foldr O.jtos  "" l
    y <- getCPUTime
    putStrLn $ " --> (O.jtos) time=" ++ show (y-x)

-- input: a lower bound (n) and a number of numbers (w)
-- runs test with numbers n..n+w-1
test' :: Integer -> Integer -> IO ()
test' n w = test ((show $ length $ show n) ++ ".." ++
                  (show $ length (show $ n+w)) ++ " digits")
                 [n..n+w-1]

-- input: an upper bound (n) and a number of numbers (w)
-- runs test with w numbers in the range 0..n-1
testsmall :: Integer -> Integer -> IO ()
testsmall    n w = test ("up to " ++ show n) $ map (`mod`n) [0..w-1]

-- same as testsmall but the range is -n+1..0
testsmallneg :: Integer -> Integer -> IO ()
testsmallneg n w = test ("down to " ++ show (-n))
                 $ map (0-) $ map (`mod`n) [0..w-1]

main :: IO ()
main = do
    testsmall 2                                            1000000
    testsmall 16                                           1000000
    testsmall 256                                          1000000
    testsmall 1024                                          500000
    testsmallneg 16                                        1000000
    testsmallneg 256                                       1000000
    testsmallneg 1024                                       500000
    test' 0                                                 500000
    test' 100000                                            500000
    test' 10000000                                          200000
    test' 100000000                                         200000
    test' 1135257894                                        200000
    test' 11352578940                                       100000
    test' 2135257894000000                                  100000
    test' 21352578940000000                                 100000
    test' 213525789400000000                                100000
    test' 2135257894000000000                               100000
    test' 213525789405555555500000                          100000
    test' 2135257894077777777000000                         100000
    test' 61352578941352578941352578941352578941352578940000 50000
    test' 613525789413525789413525412574135894137894178941352578941352578941352578941352578941352578941352578941352578941352578941352578941352578941352578940000 20000
